#include "usart.h"

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <stdlib.h>
#include <string.h>

#define RESET_MOTOR 0x7A1 //0xC35
#define TIMER_FREQ 0x10 //0xFA
#define TIMER_ENGINE_STOP 0xFF
#define SPEED1 0.4
#define SPEED2 0.2
#define SPEED3 0

volatile char command;
volatile int timer;

ISR(TIMER0_OVF_vect) {
	PORTD &= ~(1 << PD4);
}

ISR(TIMER0_COMPD_vect) {
	PORTD |= (1 << PD4);
}

ISR(TIMER1_OVF_vect) {
	if(timer == 0) {
	    OCR0B = TIMER_ENGINE_STOP;
	    PORTD &= ~(1 << PD5);
	}
	else timer--;
}

ISR(TIMER2_OVF_vect) {
    if(horn)
	    PORTD ^= (1 << PD7);
	else PORTD &= ~(1 << PD7);
}



void start_eng() {
    switch(command) {
		case '1':
			PORTD |= (1 << PD0);
			PORTD &= ~(1 << PD1);
			OCR0B = TIMER_FREQ * SPEED1;
			break;
		case '2':
			PORTD |= (1 << PD0);
			PORTD &= ~(1 << PD1);
			OCR0B = TIMER_FREQ * SPEED2;
			break;
		case '3':
			PORTD |= (1 << PD0);
			PORTD &= ~(1 << PD1);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '4':
			PORTD |= (1 << PD1);
			PORTD &= ~(1 << PD0);
			OCR0B = TIMER_FREQ * SPEED1;
			break;
		case '5':
			PORTD |= (1 << PD1);
			PORTD &= ~(1 << PD0);
			OCR0B = TIMER_FREQ * SPEED2;
			break;
		case '6':
			PORTD |= (1 << PD1);
			PORTD &= ~(1 << PD0);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '0':
		    OCR0B = TIMER_ENGINE_STOP;
			break;
	    case '8':
	        PORTD |= (1 << PD5);
	        PORTD |= (1 << PD2);
			PORTD &= ~(1 << PD3);
	        break;
	    case '9':
	    	PORTD |= (1 << PD5);
	        PORTD |= (1 << PD3);
			PORTD &= ~(1 << PD2);
	        break;
	    case '7':
	        PORTD &= ~(1 << PD5);
	        break;

	}

}

void init_eng() {
	DDRB |= (1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) | (1 << PD4) | (1 << PD5);
	PORTD = 0;

	/* timer 0 este configurat in modul FastPWM si prescaler 64 */
	TCCR0A |= (1 << WGM01) | (1 << WGM00);
	TCCR0B |= (1 << WGM02);
	TCCR0B |= (1 << CS02) | (1 << CS00);
	TIMSK0 |= (1 << OCIE0B) | (1 << TOIE0);
	
	OCR0A = TIMER_FREQ;
	OCR0B = TIMER_ENGINE_STOP;

	/* timer1 este configurat cu modul CNC, top OCR1A si prescaler 1024 */
	TCCR1A |= (1 << WGM10) | (1 << WGM11);
	TCCR1B |= (1 << WGM12) | (1 << WGM13);
	TCCR1B |= (1 << CS12) | (1 << CS10);
	TIMSK1 |= (1 << TOIE1);

	OCR1A = RESET_MOTOR;
	
}

int main()
{
	MCUCR |= (1<<JTD);
	MCUCR |= (1<<JTD);
	
 	USART0_init();
  	init_engine();
	sei();

    _delay_ms(100);
  
	while(1) {
	    activate_motor();
    }

    return 0;
}

